#ifdef WIN32
#include <windows.h>
#endif
#include <ode/ode.h>
#include <drawstuff/drawstuff.h>
  
#ifdef dDOUBLE
  #define dsDrawBox dsDrawBoxD
#endif

static dWorldID world;

static dBodyID body_box1;
static dBodyID body_box2;
dReal  box_sizes[3] = { 1.0, 1.0, 1.0 };

dJointID joint_ball;

FILE *fp = NULL;

#define STEP_SIZE 0.01
float time = 0;


 
// start simulation - set viewpoint
static void start()
{
  static float xyz[3] = { 0.f, -10.f, 10.f };
  static float hpr[3] = { 90.f, -15.f, 0.f };
 
  dsSetViewpoint( xyz, hpr );
}

 
// simulation loop
static void simLoop( int pause )
{
    // Ctl+p ꂽifɓȂ
    if (!pause)
    {
        dWorldStep( world, STEP_SIZE );
    }
 
    dsSetColor( 1.0f, 1.0f, 1.0f );
    dsDrawBox( dBodyGetPosition( body_box1 ), dBodyGetRotation( body_box1 ), box_sizes );

    dsSetColor( 0.0f, 1.0f, 0.0f );
    dsDrawBox( dBodyGetPosition( body_box2 ), dBodyGetRotation( body_box2 ), box_sizes );

    //
    //t@C_v
    //
    time += STEP_SIZE;
    fprintf( fp, "%.2f\t", time );

    const dReal* pos = dBodyGetPosition( body_box2 );
    fprintf( fp, "%.3f\t%.3f\t%.3f\t", pos[0], pos[1], pos[2] );

    // {[WCg̃tB[hobN
    dJointFeedback *fb_ball = dJointGetFeedback( joint_ball );
    fprintf( fp, "%.3f\t%.3f\t%.3f\t", fb_ball->f1[0], fb_ball->f1[1], fb_ball->f1[2] );
    fprintf( fp, "%.3f\t%.3f\t%.3f\t", fb_ball->t1[0], fb_ball->t1[1], fb_ball->t1[2] );
    fprintf( fp, "%.3f\t%.3f\t%.3f\t", fb_ball->f2[0], fb_ball->f2[1], fb_ball->f2[2] );
    fprintf( fp, "%.3f\t%.3f\t%.3f\n", fb_ball->t2[0], fb_ball->t2[1], fb_ball->t2[2] );

}
 
 
int main( int argc, char* argv[] )
{
    dInitODE();
 
    // setup pointers to drawstuff callback functions
    dsFunctions fn;
    fn.version = DS_VERSION;
    fn.start   = &start;
    fn.step    = &simLoop;
    fn.command = 0;
    fn.stop    = 0;
    fn.path_to_textures = "../drawstuff/textures";
 
    world = dWorldCreate();
    dWorldSetGravity( world, 0.0, 0.0, -9.8 );

 
    // box1 creating
    {
        body_box1 = dBodyCreate( world );
        dReal pos[3] = { 0.0, 0.0, 10.0 };
        dBodySetPosition( body_box1, pos[0], pos[1], pos[2] );
    }

    // box2 creating
    {
        body_box2 = dBodyCreate( world );
        dReal pos[3] = { 5.0, 0.0, 10.0 };
        dBodySetPosition( body_box2, pos[0], pos[1], pos[2] );
    }

    // box1̌Œ
    {
        dJointID fixed;
        fixed = dJointCreateFixed( world, 0 );
        dJointAttach( fixed, NULL, body_box1 );
        dJointSetFixed( fixed );
    }

    // {[WCg
    {
        joint_ball = dJointCreateBall( world, 0 );
        dJointSetBallAnchor( joint_ball, 0.0, 0.0, 10.0 ); // box1ƓW
        dJointAttach( joint_ball, body_box1, body_box2 );

        // {[WCg̃tB[hobN
        dJointFeedback *fb = new dJointFeedback;
        dJointSetFeedback( joint_ball, fb );
    }

    // t@Cݎw
    fp = fopen( "data.txt", "w" );
    if (fp == NULL) {
        printf( "can not file open.\n" );
        exit( 1 );
    }
    fprintf( fp, "time\t" );
    fprintf( fp, "box2.x\tbox2.y\tbox2.z\t" );
    fprintf( fp, "box1.f.x\tbox1.f.y\tbox1.f.z\t" );
    fprintf( fp, "box1.t.x\tbox1.t.y\tbox1.t.z\t" );
    fprintf( fp, "box2.f.x\tbox2.f.y\tbox2.f.z\t" );
    fprintf( fp, "box2.t.x\tbox2.t.y\tbox2.t.z\n" );

    dsSimulationLoop( argc, argv, 320, 240, &fn );

    fclose( fp );

    dWorldDestroy( world );
    dCloseODE();
    return 0;
}



